/*
 * webfilter_add.c -- add an entry to the database
 *
 * (c) 2003 Dr. Andreas Mueller, Beratung und Entwicklung
 *
 * $Id: webfilter_add.c,v 1.6 2003/06/27 11:04:28 afm Exp $
 */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <gdbm.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#define	LINELENGTH	10240

int	debug = 0;

int	main(int argc, char *argv[]) {
	char		*dbname, *hostname, *comment = NULL;
	int		c, i, l, off, rc, xc = EXIT_SUCCESS, keysize;
	GDBM_FILE	db;
	char		el[LINELENGTH];
	datum		key, entry;

	/* parse command line						*/
	while (EOF != (c = getopt(argc, argv, "dc:")))
		switch (c) {
		case 'd':
			debug++;
			break;
		case 'c':
			comment = optarg;
			break;
		}

	/* check for database name and hostname				*/
	if ((argc - 1) < (optind + 2)) {
		fprintf(stderr,
			"%s: must specify database, host an categories\n",
			argv[0]);
		exit(EXIT_FAILURE);
	}
	dbname = argv[optind];
	hostname = argv[optind + 1];
	keysize = strlen(hostname) + 1;

	/* open database						*/
	if (NULL == (db = gdbm_open(dbname, 0, O_RDWR, 0666, NULL))) {
		fprintf(stderr, "%s: cannot open database %s: %s (%d)\n",
			argv[0], dbname, strerror(errno), errno);
		exit(EXIT_FAILURE);
	}

	/* setup key							*/
	key.dptr = hostname;
	key.dsize = keysize;
	if (debug)
		fprintf(stderr, "%s:%d: new key: %s\n", __FILE__, __LINE__,
			hostname);

	/* setup entry;							*/
	for (i = optind + 2, off = 0; i < argc; i++) {
		l = strlen(argv[i]);
		if (l > 0) {
			if (debug)
				fprintf(stderr, "%s:%d: adding category %s\n",
					__FILE__, __LINE__, argv[i]);
			memcpy(el + off, argv[i], l + 1);
			off += l + 1;
		}
	}
	entry.dptr = el;
	entry.dsize = off;
	if (0 != (rc = gdbm_store(db, key, entry, GDBM_REPLACE))) {
		fprintf(stderr, "%s: cannot store %s: %s\n", argv[0], hostname,
			(rc < 0) ? strerror(errno) : "entry exists");
		xc = EXIT_FAILURE;
	}

	/* if we have a comment, we should also add a comment entry	*/
	if (comment) {
		key.dsize = keysize + 1;
		key.dptr = (char *)alloca(key.dsize);
		strcpy((char *)key.dptr, hostname);
		strcat((char *)key.dptr, "#");
		entry.dptr = comment;
		entry.dsize = strlen(comment) + 1;
		gdbm_store(db, key, entry, GDBM_REPLACE);
	}

	/* close database						*/
	gdbm_close(db);

	exit(xc);
}
