/*
 * webfilter_check.c -- check a host against the a database and category
 *
 * (c) 2003 Dr. Andreas Mueller, Beratung und Entwicklung
 *
 * $Id: webfilter_check.c,v 1.4 2003/06/27 11:04:28 afm Exp $
 */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <gdbm.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

int	debug = 0;

int	main(int argc, char *argv[]) {
	int		c, i, off, xc = EXIT_SUCCESS;
	char		*dbname, *hostname, *p;
	GDBM_FILE	db;
	datum		key, entry;

	/* parse command line						*/
	while (EOF != (c = getopt(argc, argv, "d")))
		switch (c) {
		case 'd':
			debug++;
			break;
		}

	/* look for database name and hostname				*/
	if ((argc - 1) < (optind + 2)) {
		fprintf(stderr, "%s: not enough arguments, must specify db, "
			"host and cateogry\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	dbname = argv[optind];
	hostname = argv[optind + 1];

	/* open database						*/
	if (NULL == (db = gdbm_open(dbname, 0, O_RDONLY, 0666, NULL))) {
		fprintf(stderr, "%s: cannot open database %s: %s (%d)\n",
			argv[0], dbname, strerror(errno), errno);
		exit(EXIT_FAILURE);
	}

	/* retrieve node for host					*/
	key.dptr = hostname;
	key.dsize = strlen(hostname) + 1;
	entry = gdbm_fetch(db, key);
	if (NULL == entry.dptr) {
		if (debug)
			fprintf(stderr, "%s:%d: no entry for %s\n", __FILE__,
				__LINE__, hostname);
		goto cleanup;
	}

	/* verify host name in list					*/
	p = entry.dptr;
	for (i = optind + 2; i < argc; i++) {
		if (debug)
			fprintf(stderr, "%s:%d: checking for category %s\n",
				__FILE__, __LINE__, argv[i]);
		for (off = 0; off < entry.dsize; off += strlen(p + off) + 1) {
			if (debug)
				fprintf(stderr, "%s:%d: check %s\n",
					__FILE__, __LINE__, p + off);
			if (0 == strcmp(p + off, argv[i])) {
				xc = EXIT_FAILURE;
				goto cleanup;
			}
		}
	}

	/* close database						*/
cleanup:
	gdbm_close(db);

	if (debug)
		fprintf(stderr, "%s:%d: %s is %s\n", __FILE__, __LINE__,
			hostname, (xc) ? "not ok" : "ok");
	exit(xc);
}
