/*
 * webfilter_delete.c -- delete an entry from the database
 *
 * (c) 2003 Dr. Andreas Mueller, Beratung und Entwicklung
 */
#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <gdbm.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

int	debug = 0;

int	main(int argc, char *argv[]) {
	int		c, xc = EXIT_SUCCESS, rc, keysize;
	GDBM_FILE	db;
	char		*dbname, *hostname;
	datum		key;

	/* parse command line						*/
	while (EOF != (c = getopt(argc, argv, "d")))
		switch (c) {
		case 'd':
			debug++;
			break;
		}

	/* check for database name					*/
	if ((argc - 1) < (optind + 1)) {
		fprintf(stderr, "%s: must specify database name and host\n",
			argv[0]);
		exit(EXIT_SUCCESS);
	}
	dbname = argv[optind];
	
	/* open database						*/
	if (NULL == (db = gdbm_open(dbname, 0, O_RDWR, 0666, NULL))) {
		fprintf(stderr, "%s: cannot open database %s: %s (%d)\n",
			argv[0], dbname, strerror(errno), errno);
		exit(EXIT_FAILURE);
	}

	/* setup the key for the given hostname				*/
	hostname = argv[optind + 1];
	keysize = strlen(hostname) + 1;

	/* construct the key for the categories key			*/
	key.dptr = hostname;
	key.dsize = keysize;

	/* delete the entry						*/
	if (0 != (rc = gdbm_delete(db, key))) {
		fprintf(stderr, "%s: cannot delete entry %s: %s\n", argv[0],
			hostname,
			(rc < 0) ? strerror(errno) : "does not exist");
		xc = EXIT_FAILURE;
	}

	/* check for a comment entry, which we should also delete	*/
	key.dsize = keysize + 1;
	key.dptr = alloca(key.dsize);
	strcpy((char *)key.dptr, hostname);
	strcat((char *)key.dptr, "#");
	gdbm_delete(db, key);

	/* close database						*/
	gdbm_close(db);
	exit(xc);
}
